'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

import os, pygame

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))

def filepath(filename):
  '''Determine the path to a file in the data directory.
  '''
  return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
  '''Open a file in the data directory.

  "mode" is passed as the second arg to open().
  '''
  return open(os.path.join(data_dir, filename), mode)


def load_image(name, colorkey=None):
  fullname = os.path.join('data', 'images', name)
  try:
    image = pygame.image.load(fullname)
  except pygame.error, message:
    print 'Cannot load image:', name
    raise SystemExit, message
  if image.get_alpha() is None:
    image = image.convert()
  else:
    image = image.convert_alpha()
  if colorkey is not None:
    if colorkey is -1:
      colorkey = image.get_at((0,0))
    image.set_colorkey(colorkey, pygame.locals.RLEACCEL)
  return image, image.get_rect()

def load_sound(name):
  fullname = os.path.join('data', 'sounds', name)
  try:
    sound = pygame.mixer.Sound(fullname)
  except pygame.error, message:
    print 'Cannot load sound:', name
    raise SystemExit, message
  return sound
