'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

import data, os, sys, pygame
from pygame.locals import *
from baseobjects import *
from objects import *
from chrome import *
import globals
from globals import *
from collections import deque
from time import time
from menu import cMenu, EVENT_CHANGE_STATE


def main():
  #pygame.init() #done in globals.py

  pygame.display.set_caption('PyWeek Game')
  pygame.mouse.set_visible(0)
  title_page = data.load_image('title-page.png')[0]
  SCREEN.blit(title_page, SCREENRECT)
  pygame.display.flip()

  menu = cMenu(550, 100, 20, 25, 'vertical', 10, SCREEN,
              [
               ('Start', 'start', None),
               ('Instructions', 'instructions', None),
               ('Quit' , 'quit' , None),
              ])
  menu.set_alignment('center', 'center')
  state = 0
  prev_state = -1
  rect_list = []

  # Ignore mouse motion (greatly reduces resources when not needed)
  pygame.event.set_blocked(pygame.MOUSEMOTION)

  while 1:
    # Check if the state has changed, if it has, then post a user event to
    # the queue to force the menu to be shown at least once
    if prev_state != state:
      pygame.event.post(pygame.event.Event(EVENT_CHANGE_STATE, key = 0))
      prev_state = state

    event = pygame.event.wait()
    if event.type == pygame.KEYDOWN or event.type == EVENT_CHANGE_STATE:
      if state == 0:
        rect_list, state = menu.update(event, state)
      elif state == 'start':
        level_menu()
        title_page = data.load_image('title-page.png')[0]
        SCREEN.blit(title_page, SCREENRECT)
        pygame.display.flip()
        state = 0
      elif state == 'instructions':
        instructions = data.load_image('instructions.png')[0]
        SCREEN.blit(instructions, SCREENRECT)
        pygame.display.flip()
        state = 0
      elif state == 'quit':
        pygame.quit()
        sys.exit()
    elif event.type == pygame.QUIT:
      pygame.quit()
      sys.exit()
    pygame.display.update(rect_list)

def level_menu():
  title_page = data.load_image('title-page.png')[0]
  SCREEN.blit(title_page, SCREENRECT)
  pygame.display.flip()

  menu = cMenu(550, 100, 20, 25, 'vertical', 10, SCREEN,
              [
               ('Level 1', 1, None),
               ('Main Menu', 'mainmenu', None),
               #('Level 2', 2, None),
              ])
  menu.set_alignment('center', 'center')
  state = 0
  prev_state = -1
  rect_list = []

  # Ignore mouse motion (greatly reduces resources when not needed)
  pygame.event.set_blocked(pygame.MOUSEMOTION)

  while 1:
    # Check if the state has changed, if it has, then post a user event to
    # the queue to force the menu to be shown at least once
    if prev_state != state:
      pygame.event.post(pygame.event.Event(EVENT_CHANGE_STATE, key = K_DOWN))
      prev_state = state

    event = pygame.event.wait()
    if event.type == pygame.KEYDOWN or event.type == EVENT_CHANGE_STATE:
      if state == 0:
        rect_list, state = menu.update(event, state)
      elif state == 'mainmenu':
        return
      elif state == 'quit':
        pygame.quit()
        sys.exit()
      else:
        level = state
        requested_state = play(level)
        if requested_state == 'restart':
          pygame.event.post(pygame.event.Event(EVENT_CHANGE_STATE, key = 0))
        elif requested_state == 'nextlevel':
          if level == len(level_waves):
            #finished last level, go back to levels menu
            title_page = data.load_image('title-page.png')[0]
            SCREEN.blit(title_page, SCREENRECT)
            pygame.display.flip()
            state = 0
          else:
            state = level + 1
        elif requested_state == 'levels':
          title_page = data.load_image('title-page.png')[0]
          SCREEN.blit(title_page, SCREENRECT)
          pygame.display.flip()
          state = 0
        else:
          return requested_state
    elif event.type == pygame.QUIT:
      pygame.quit()
      sys.exit()
    pygame.display.update(rect_list)


def play(level):

  clock = pygame.time.Clock()

  setup_level(level)
  SCREEN.blit(BG, BG.get_rect())
  pygame.display.flip()

  crosshair = Crosshair()
  MISC.add(crosshair)

  healthbar = HealthBar()

  player = Player(WIDTH/2, HEIGHT/2)
  for x in range(300,340, 8):
    Human(x, 100)

  #fps_disp = TextSprite()
  #MISC.add(fps_disp)
  count = 0
  wavenum = 0
  wavecount = 0
  paused = False
  start = time()

  wavequeue = deque()

  pygame.event.set_blocked(pygame.MOUSEMOTION)
  while 1:
    for event in pygame.event.get():
      if event.type == QUIT:
        pygame.quit()
        sys.exit()
      elif event.type == KEYDOWN:
        if KEYS.has_key(event.key):
          player.bitmask |= KEYS[event.key]
        elif event.key == K_p or event.key == K_ESCAPE:
          requested_state = pause_menu()
          if requested_state:
            return requested_state
      elif event.type == KEYUP and KEYS.has_key(event.key):
        player.bitmask &= ~KEYS[event.key]
      elif event.type == MOUSEBUTTONDOWN:
        if event.button == 1:
          player.bitmask |= KEYS[M_1]
        elif event.button == 3:
          player.bitmask |= KEYS[M_3]
      elif event.type == MOUSEBUTTONUP:
        if event.button == 1:
          player.bitmask &= ~KEYS[M_1]
        if event.button == 3:
          player.bitmask &= ~KEYS[M_3]
      elif event.type == EVENT_CHANGE_STATE: # and event.state == 'nextlevel'
        return event.state


    globals.SOUNDCOUNT -= (random() * 4) / FPS
    if globals.SOUNDCOUNT < 0: globals.SOUNDCOUNT = 0

    x, y = pygame.mouse.get_pos()
    MOUSEPOS[0] = x
    MOUSEPOS[1] = y
    crosshair.rect.center = x,y

    if not paused:
      
      wavecount, wavenum, wavesdone = level_waves[level](wavecount, wavenum, wavequeue)

      if healthbar.health == 0:
        notify("You Lost - You've become a mutant!")
        pygame.time.wait(4000)
        for group in ALLGROUPS:
          group.empty()
        return 'levels'
      elif len(HUMANS.sprites()) == 0 and \
           len(MUTATORS.sprites()) == 0 and \
           len(UNMUTATORS.sprites()) == 0:
        MUTANTS.draw(SCREEN)
        PLAYERS.draw(SCREEN)
        pygame.display.update()
        notify("You Lost - The people you were protecting are all mutants now!")
        pygame.time.wait(4000)
        for group in ALLGROUPS:
          group.empty()
        return 'levels'

      if wavesdone:
        if len(wavequeue) == 0 and \
           len(MUTANTS.sprites()) == 0 and \
           len(DEAD.sprites()) == 0 and \
           len(MUTATORS.sprites()) == 0 and \
           len(UNMUTATORS.sprites()) == 0:
          notify("Level Passed - saved " + str(len(HUMANS.sprites())) + " out of 5 people")
          pygame.time.wait(4000)
          for group in ALLGROUPS:
            group.empty()
          pygame.event.post(pygame.event.Event(EVENT_CHANGE_STATE, state='nextlevel'))
        

      # use spritecollideany instead?
      player_mutant_collisions = pygame.sprite.spritecollide(
          player, MUTANTS, False, pygame.sprite.collide_rect_ratio(0.75))
      player_dead_mutant_collisions = pygame.sprite.spritecollide(
          player, DEAD, False, pygame.sprite.collide_rect_ratio(0.75))
      if len(player_mutant_collisions) > 0:
        healthbar.reduce_health(40.0/FPS)
      elif len(player_dead_mutant_collisions) > 0:
        healthbar.reduce_health(5.0/FPS)
      else:
        healthbar.increase_health(0.75/FPS)

      if healthbar.health == 0:
        player.mutate()


      # groupcollide uses rect, not colrect
      human_mutant_collisions = pygame.sprite.groupcollide(HUMANS, MUTANTS, False, False)
      #TODO finish!!!!!!!!!!! - for better FPS, if i need it
      '''
      for human in HUMANS.sprites():
        if pygame.sprite.spritecollideany(human, MUTANTS):
          human.start_mutation( #TODO finish - need FourEyesMutants group, etc
      '''
      for human in human_mutant_collisions:
        if len(human_mutant_collisions[human]) > 0:
          mutant = human_mutant_collisions[human][0]
          human.start_mutation(mutant.__class__)

      for sprites in [DEAD, HUMANS, UNMUTATORS, MUTANTS, MUTATORS, PLAYERS, MISC]:
        sprites.clear(SCREEN, BG)
      dirty = []
      for sprites in [DEAD, HUMANS, UNMUTATORS, MUTANTS, MUTATORS, PLAYERS, MISC]:
        sprites.update()
        dirty += sprites.draw(SCREEN)
      pygame.display.update(dirty)


    clock.tick(FPS)

    '''
    count += 1
    if paused:
      count -= 1
    elif count == FPS / 5:
      count = 0
      elapsed = time() - start
      fps = FPS / (elapsed*5)
      fps_disp.set_text(str(int(fps)))
      start = time()
    '''



def pause_menu():
  menu_surf = pygame.Surface((WIDTH/2, HEIGHT * 3/4))
  menu_surf.fill((255,255,255))
  menu_rect = menu_surf.get_rect().move((WIDTH/4, HEIGHT/8))

  menu = cMenu(0, 0, 20, 15, 'vertical', 10, menu_surf,
              [('Continue', 'continue', None),
               ('Restart', 'restart', None),
               ('Levels', 'levels', None),
               ('Main Menu', 'mainmenu', None),
               ('Quit' , 'quit' , None)])
  menu.set_center(True, True)
  menu.set_alignment('center', 'center')
  state = 0
  prev_state = -1

  while 1:
    # Check if the state has changed, if it has, then post a user event to
    # the queue to force the menu to be shown at least once
    if prev_state != state:
      pygame.event.post(pygame.event.Event(EVENT_CHANGE_STATE, key = 0))
      prev_state = state

    event = pygame.event.wait()
    if event.type == pygame.KEYDOWN and \
        (event.key == K_p or event.key == K_ESCAPE):
       state = 'continue'
    elif event.type == pygame.KEYDOWN or event.type == EVENT_CHANGE_STATE:
      if state == 0:
        rect_list, state = menu.update(event, state)
      elif state == 'continue':
        SCREEN.blit(BG, BG.get_rect())
        pygame.display.flip()
        return
      elif state == 'mainmenu' or state == 'levels' or state == 'restart':
        for group in ALLGROUPS:
          group.empty()
        return state
      elif state == 'quit':
        pygame.quit()
        sys.exit()
    elif event.type == pygame.QUIT:
      pygame.quit()
      sys.exit()

    SCREEN.blit(menu_surf, menu_rect)
    pygame.display.update(menu_rect)



def setup_level(level):
  #BG.fill((150, 220, 250))
  bg_image = data.load_image('level'+str(level)+'-bg.png', None)[0]
  level_image = data.load_image('level'+str(level)+'.png', None)[0]

  BG.blit(bg_image, BGIR)
  BG.blit(level_image, BGIR)
  globals.BGIR = level_image.get_rect().move(-XOFF, -YOFF)
  globals.BGIA = pygame.surfarray.array_alpha(level_image)


def level1_wave(wavecount, wavenum, wavequeue):
  wavesdone = wavenum >= 13
  wavecount += 1
  if not wavesdone:
    if wavecount > FPS * 5 + wavenum * 8 or \
       (len(wavequeue) == 0 and \
        len(MUTANTS.sprites()) == 0 and \
        len(DEAD.sprites()) == 0):
      wavecount = 0
      wavenum += 2
      wave(wavenum, wavequeue)
  if wavecount % (FPS/2) == 0 and len(wavequeue) > 0:
    if wavequeue.popleft() % 2:
      if random() > 0.7:
        TwoEyes(WIDTH-25, 10)
      else:
        FourEyes(WIDTH-25, 10)
    else:
      if random() > 0.7:
        TwoEyes(10, -40)
      else:
        FourEyes(10, -40)
  return (wavecount, wavenum, wavesdone)

def level2_wave(wavecount, wavenum, wavequeue):
  wavesdone = wavenum > 15
  wavecount += 1
  if not wavesdone:
    if wavecount > FPS * 5 + wavenum * 8 or \
       (len(wavequeue) == 0 and \
        len(MUTANTS.sprites()) == 0 and \
        len(DEAD.sprites()) == 0):
      wavecount = 0
      wavenum += 2
      wave(wavenum, wavequeue)
  if wavecount % (FPS/2) == 0 and len(wavequeue) > 0:
    if wavequeue.popleft() % 2:
      FourEyes(WIDTH-25, 10)
    else:
      FourEyes(10, -40)
  return (wavecount, wavenum, wavesdone)


def wave(num, wavequeue):
  for i in range(num):
    wavequeue.append(i)


level_waves = {
  1: level1_wave,
  #'level2': level2_wave,
}


def notify(msg):
  ntf = TextSprite(20)
  ntf.set_text(msg)
  ntfbg = pygame.Surface((ntf.rect.width + 20, ntf.rect.height + 20))

  ntfbgrect = ntfbg.get_rect()
  ntfbgrect.center = SCREENRECT.center

  ntfbg.fill((255,255,255))
  ntfbg.blit(ntf.image, ntf.rect.move((10, 10)))
  SCREEN.blit(ntfbg, ntfbgrect)

  pygame.display.update(ntfbgrect)

