import data, os, sys, pygame
from pygame.locals import *
import globals
from globals import *
from euclid import Vector2
from random import random
from baseobjects import *



class Player(Character):
  bullet_sound = data.load_sound('shot.wav')
  anti_bullet_sound = data.load_sound('anti-bullet.wav')

  frame_images, flipped_frame_images, frame_step = Character.init_frames({
    'default': {
      'images': ['player-stand.png']
    },
    'stand': {
      'images': [ 'player-stand.png' ]
    },
    'walk': {
      'images': ["player-run%(#)d.png" % {'#': x} for x in range(1,11)],
      'anim_time': 0.5
    },
  }, (255,255,255))


  maxup    = 16
  maxleft  = 9
  maxright = 9


  def __init__(self, x=100, y=100):
    self.bitmask = 0
    self.scount = 0
    self.action = self.stand
    Character.__init__(self, x, y) 
    PLAYERS.add(self)
    Gun(self)


  def update(self):
    self.scount += 1
    if self.bitmask & KEYS[M_3] and self.scount > FPS/6:
      self.anti_bullet_sound.play()
      self.scount = 0
      MISC.add(AntiBullet(self.rect.center, MOUSEPOS))
    elif self.bitmask & KEYS[M_1] and self.scount > FPS/6:
      self.bullet_sound.play()
      self.scount = 0
      MISC.add(Bullet(self.rect.center, MOUSEPOS))
    if MOUSEPOS[0] > self.rect.centerx:
      self.dir = K_RIGHT
    else:
      self.dir = K_LEFT
    Character.update(self)


  def walk(self):
    self.frame_update('walk')

    self.speed.x = 0
    if self.bitmask & KEYS[K_LEFT]:
      self.speed.x = -self.maxleft
    elif self.bitmask & KEYS[K_RIGHT]:
      self.speed.x = self.maxright

    if self.bitmask == 0:
      self.action = self.stand
    elif self.bitmask & KEYS[K_UP]:
      self.action = self.jump


  def stand(self):
    self.frame_update('stand')
    self.speed.x = 0
    if self.bitmask & (KEYS[K_LEFT] | KEYS[K_RIGHT]):
      self.action = self.walk
    elif self.bitmask & KEYS[K_UP]:
      self.action = self.jump


  def jump(self):
    if self.bitmask & KEYS[K_UP] and self.colrect_bg_collide('bottom', 4):
      self.f += DIR.up * self.m * self.maxup
    elif self.bitmask == 0:
      self.action = self.stand
    else:
      self.action = self.walk

  def mutate(self):
    self.action = self.mutate
    self.image = data.load_image('player_mutant.png')[0]
    #self.rect.left -= 10



class Mutant(Character):
  die_sound = data.load_sound('splat.wav')

  def __init__(self, x=300, y=100):
    self.action = self.walk
    self.wcount = 0
    #self.pcount = 0
    Character.__init__(self, x, y)
    MUTANTS.add(self)


  # this is for astar, but i've left it unfinished. not priority
  # TODO finish
  """
  def chase(self):
    def neighbors(self, y, x):
      neighbors = []
      if self.colrect_bg_collide('bottom', 1):
        # on the ground
        self.bg_collide(x, y)
      else:
        # in the air
        self.bg_collide(x, y)

    #TODO make targetRects global, and only generate once per loop
    if self.pcount > FPS:
      self.pcount = 0
      pRects = [x.rect for x in PLAYERS.sprites()]
      hRects = [x.rect for x in HUMANS.sprites()]
      targetRects = pRects + hRects
      i = self.visrect.collidelist(targetRects)
      if i == -1:
        #move towards center
        if self.rect.right > SCREENRECT.centerx + 100 + random()*100:
          self.dir = K_LEFT
          self.speed.x = -self.maxleft
        elif self.rect.left < SCREENRECT.centerx - 100 - random()*100:
          self.dir = K_RIGHT
          self.speed.x = self.maxright
      else:
        target = targetRects[i]
        self.path = astar( self.rect.midbottom, neighbors, 
                           (target.centerx, target.centery), 0, cost, estimate)
            
    else:
      self.path
  """


  def walk(self, jump_prob=0.1):
    #TODO make targetRects global, and only generate once per loop
    self.wcount += 1
    if self.wcount > FPS / 2:
      self.wcount = 0
      pRects = [x.rect for x in PLAYERS.sprites()]
      hRects = [x.rect for x in HUMANS.sprites()]
      targetRects = pRects + hRects
      i = self.visrect.collidelist(targetRects)
      #TODO A* or similar movement
      if i == -1:
        #move towards center
        if self.rect.right > SCREENRECT.centerx + 100 + random()*100:
          self.dir = K_LEFT
          self.speed.x = -self.maxleft
        elif self.rect.left < SCREENRECT.centerx - 100 - random()*100:
          self.dir = K_RIGHT
          self.speed.x = self.maxright
      else:
        target = targetRects[i]
        if self.rect.centerx > target.centerx:
          self.dir = K_LEFT
          self.speed.x = -self.maxleft
        else:
          self.dir = K_RIGHT
          self.speed.x = self.maxright

    if random() > (1-jump_prob):
      self.action = self.jump


  def jump(self):
    self.action = self.walk
    if self.colrect_bg_collide('bottom', 2):
      self.f += DIR.up * self.m * self.maxup
      return True

  def start_dying(self):
    self.die_sound.play()
    MUTANTS.remove(self)
    DEAD.add(self)
    self.speed.x *= 0.5
    self.old_image = self.image.copy()
    self.dcount = 9
    self.action = self.die

  def die(self):
    if self.dcount > FPS:
      self.kill()
    else:
      self.dcount += 1
      self.image = pygame.transform.rotate(self.old_image, 90 * self.dcount/FPS)



class FourEyes(Mutant):

  jump_sound = data.load_sound('foureyes-jump.wav')

  frame_images, flipped_frame_images, frame_step = Character.init_frames({
    'default': {
      'images': ['mutant.png']
    },
  })

  maxup = 10
  maxleft  = 4
  maxirght = 4

  def __init__(self, x=300, y=100):
    Mutant.__init__(self, x, y)
    FOUREYES.add(self)


  def jump(self):
    jumped = Mutant.jump(self)
    if jumped and globals.SOUNDCOUNT < 4:
      globals.SOUNDCOUNT += 1
      self.jump_sound.play()


  def walk(self):
    Mutant.walk(self, 0.2)

class TwoEyes(FourEyes):

  maxup = 14
  maxleft  = 8
  maxirght = 8

  frame_images, flipped_frame_images, frame_step = Character.init_frames({
    'default': {
      'images': ['tentacles2.png']
    },
    'walk': {
      'images': ["tentacles%(#)d.png" % {'#': x} for x in [1,2,3,2]],
      'anim_time': 0.4
    },
  })

  def __init__(self, x=300, y=100):
    Mutant.__init__(self, x, y)

  def walk(self):
    self.frame_update('walk')
    Mutant.walk(self, 0.3)


class Human(Character):

  scream_sound = data.load_sound('scream.wav')
   
  frame_images, flipped_frame_images, frame_step = Character.init_frames({
    'default': {
      'images': ['walk__15.png']
    },
    'walk': {
      # http://opengameart.org/content/plataform-walk-jump-animation-side-view
      'images': ["walk__%(#)02d.png" % {'#': x} for x in range(1,32)],
      'anim_time': 0.5
    },
    'jump': {
      # http://opengameart.org/content/plataform-walk-jump-animation-side-view
      'images': ["jump__%(#)02d.png" % {'#': x} for x in range(1,23)],
      'anim_time': 0.5
    },
    'mutate': {
      'images': ["mutating%(#)d.png" % {'#': x} for x in [1,2,3,4,3,2]],
      'anim_time': 0.3
    },
    'unmutate': {
      'images': ["unmutating%(#)d.png" % {'#': x} for x in [1,2,3,4,3,2]],
      'anim_time': 0.5
    },
  }, (255,0,255))

  maxleft = 6
  maxright = 6


  def __init__(self, x=300, y=100):
    self.action = self.walk
    self.wcount = 0
    self.jcount = 0
    self.mcount = 0
    Character.__init__(self, x, y)
    HUMANS.add(self)


  def walk(self, jump_prob = 0.05):
    self.frame_update('walk')
    #TODO make targetRects global, and only generate once per loop
    self.wcount += 1
    if self.wcount > FPS / 10:
      self.wcount = 0
      mrects = [x.rect for x in MUTANTS.sprites()]
      i = self.visrect.collidelist(mrects)
      #TODO A* or similar movement
      if i == -1:
        #move towards center
        if self.rect.right < SCREENRECT.centerx - random()*100:
          self.dir = K_RIGHT
          self.speed.x = self.maxright
        elif self.rect.left > SCREENRECT.centerx + random()*100:
          self.dir = K_LEFT
          self.speed.x = -self.maxleft
      else:
        mutant = mrects[i]
        # flee from the mutant
        if self.rect.centerx > mutant.centerx:
          self.dir = K_RIGHT
          self.speed.x = self.maxright
          if self.colrect_bg_collide('right', 1, -self.maxright*1.8):
            self.action = self.jump
        else:
          self.dir = K_LEFT
          self.speed.x = -self.maxleft

    if random() > (1-jump_prob):
      self.action = self.jump


  def jump(self):
    self.frame_update('jump')
    if self.colrect_bg_collide('bottom', 3):
      self.f += DIR.up * self.m * self.maxup
    self.jcount += 1
    if self.jcount > FPS/2:
      self.jcount = 0
      self.action = self.walk


  def start_mutation(self, type):
    if self.mcount == 0:
      self.speed.x = 0
      self.action = self.mutate
      HUMANS.remove(self)
      MUTATORS.add(self)
      self.mutant_type = type
      self.scream_sound.play()

  def mutate(self):
    self.frame_update('mutate')
    self.mcount += 1
    if self.mcount > FPS * 3:
      self.mutant_type(self.rect.centerx, self.rect.centery-19)
      self.kill()


  def reverse_mutation(self):
    MUTATORS.remove(self)
    UNMUTATORS.add(self)
    self.mcount = FPS * 2
    self.action = self.unmutate

  def unmutate(self):
    self.frame_update('unmutate')
    self.mcount -= 1
    if self.mcount == 0:
      UNMUTATORS.remove(self)
      HUMANS.add(self)
      self.action = self.walk



class Bullet(GameObject):
  speed = 30
  image = data.load_image('bullet.png')[0]
  
  def __init__(self, xy, xy2):
    x, y = xy
    x2, y2 = xy2
    GameObject.__init__(self, x, y) 
    self.direction = Vector2(x2-x, y2-y).normalized()
    self.v = self.direction * self.speed # not doing anthing with this...
    self.vstep = self.direction * 8
    self.vremstep = self.direction * (self.speed % 8)

  def update(self):
    times = self.speed / 8
    for i in range(1, times+1):
      self.rect.move_ip((self.vstep.x, self.vstep.y))
      if self.bg_collide(self.rect.centerx, self.rect.centery):
        self.kill()
        return
      elif (i+1) % 2 == 0:
        # repace with rect.collidelist? faster?
        collided_mutants = pygame.sprite.spritecollide(self, MUTANTS, False)
        if len(collided_mutants) > 0:
          collided_mutants[0].start_dying()
          self.kill()
          return
    self.rect.move_ip((self.vremstep.x, self.vremstep.y))
    if self.bg_collide(self.rect.centerx, self.rect.centery):
      self.kill()
      return

class AntiBullet(Bullet):
  image = data.load_image('anti-bullet.png')[0]

  def __init__(self, xy, xy2):
    Bullet.__init__(self, xy, xy2)

  def update(self):
    times = self.speed / 8
    for i in range(1, times+1):
      self.rect.move_ip((self.vstep.x, self.vstep.y))
      if self.bg_collide(self.rect.centerx, self.rect.centery):
        self.kill()
        return
      elif (i+1) % 2 == 0:
        # repace with rect.collidelist? faster?
        collided_mutators = pygame.sprite.spritecollide(self, MUTATORS, False)
        if len(collided_mutators) > 0:
          collided_mutators[0].reverse_mutation()
          self.kill()
          return
    self.rect.move_ip((self.vremstep.x, self.vremstep.y))
    if self.bg_collide(self.rect.centerx, self.rect.centery):
      self.kill()
      return


class Gun(GameObject):
  images = [data.load_image('gun'+str(i)+'.png')[0] for i in range(1,4)]
  flipped_images = [pygame.transform.flip(img, True, False) for img in images]
  image = images[1]

  def __init__(self, player):
    self.player = player
    self.rect = player.rect
    GameObject.__init__(self, player.rect.centerx, player.rect.centery)
    MISC.add(self)

  def update(self):
    self.rect = self.player.rect
    slope = (MOUSEPOS[1] - self.rect.centery + 0.01) / abs(0.01 + MOUSEPOS[0] - self.rect.centerx)
    if self.player.dir == K_LEFT:
      images = self.images
    else:
      images = self.flipped_images
    if slope < -0.6:
      self.image = images[0]
    elif slope > 0.6:
      self.image = images[2]
    else:
      self.image = images[1]


class Crosshair(GameObject):

  # credit: thttp://opengameart.org/content/20-crosshairs-for-re
  image = data.load_image('crosshair.png')[0]
  rect = image.get_rect()

  def __init__(self):
    GameObject.__init__(self) 
